@@ -30,6 +30,7 @@
 
 #include "avcodec.h"
 #include "get_bits.h"
+#include "idctdsp.h"
 #include "internal.h"
 #include "simple_idct.h"
 #include "proresdec.h"
@@ -50,6 +51,7 @@ static av_cold int decode_init(AVCodecContext *avctx)
     avctx->bits_per_raw_sample = 10;
 
     ff_dsputil_init(&ctx->dsp, avctx);
+    ff_blockdsp_init(&ctx->bdsp, avctx);
     ff_proresdsp_init(&ctx->prodsp, avctx);
 
     ff_init_scantable_permutation(idct_permutation,
@@ -182,8 +184,7 @@ static int decode_picture_header(AVCodecContext *avctx, const uint8_t *buf, cons
 
     if (ctx->slice_count != slice_count || !ctx->slices) {
         av_freep(&ctx->slices);
-        ctx->slice_count = 0;
-        ctx->slices = av_mallocz(slice_count * sizeof(*ctx->slices));
+        ctx->slices = av_mallocz_array(slice_count, sizeof(*ctx->slices));
         if (!ctx->slices)
             return AVERROR(ENOMEM);
         ctx->slice_count = slice_count;
@@ -367,7 +368,7 @@ static int decode_slice_luma(AVCodecContext *avctx, SliceContext *slice,
     int ret;
 
     for (i = 0; i < blocks_per_slice; i++)
-        ctx->dsp.clear_block(blocks+(i<<6));
+        ctx->bdsp.clear_block(blocks+(i<<6));
 
     init_get_bits(&gb, buf, buf_size << 3);
 
@@ -400,7 +401,7 @@ static int decode_slice_chroma(AVCodecContext *avctx, SliceContext *slice,
     int ret;
 
     for (i = 0; i < blocks_per_slice; i++)
-        ctx->dsp.clear_block(blocks+(i<<6));
+        ctx->bdsp.clear_block(blocks+(i<<6));
 
     init_get_bits(&gb, buf, buf_size << 3);
 
@@ -479,7 +480,7 @@ static void decode_slice_alpha(ProresContext *ctx,
     int16_t *block;
 
     for (i = 0; i < blocks_per_slice<<2; i++)
-        ctx->dsp.clear_block(blocks+(i<<6));
+        ctx->bdsp.clear_block(blocks+(i<<6));
 
     init_get_bits(&gb, buf, buf_size << 3);
 
